"""Init the source files from the configured git url."""

from . import DodoCommand
import argparse


class Command(DodoCommand):  # noqa
    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            'git_args',
            nargs=argparse.REMAINDER
        )

    def handle_imp(  # noqa
        self, *args, **kwargs
    ):
        self.runcmd(
            [
                "git",
            ] + kwargs['git_args'],
            cwd=self.get_config("/ROOT/src_dir")
        )



# noqa
import argparse
from dodo_commands.extra.standard_commands import DodoCommand
from dodo_commands.framework.util import remove_trailing_dashes


class Command(DodoCommand):  # noqa
    help = ""
    decorators = ["docker"]
    docker_options = [
        '--name=tape',
    ]

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            'tape_args',
            nargs=argparse.REMAINDER
        )

    def handle_imp(self, tape_args, **kwargs):  # noqa
        self.runcmd(
            [
                self.get_config("/TAPE/tape"),
                self.get_config("/TAPE/bundle_file"),
            ] + remove_trailing_dashes(tape_args)
        )



# noqa
from dodo_commands.extra.standard_commands import DodoCommand
from dodo_commands.framework import call_command
from plumbum import local
from plumbum.cmd import git


class Command(DodoCommand):  # noqa
    help = ""
    decorators = []

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument('branch')

    def handle_imp(self, branch, **kwargs):  # noqa
        src_dir = self.get_config("/ROOT/src_dir")
        with local.cwd(src_dir):
            has_local_changes = git("diff")

        if has_local_changes:
            self.runcmd(
                ["git", "stash", "save", "create_branch_" + branch], cwd=src_dir)
        self.runcmd(["git", "checkout", "dev"], cwd=src_dir)
        self.runcmd(["git", "pull"], cwd=src_dir)
        self.runcmd(["git", "checkout", "-b", branch], cwd=src_dir)
        with local.cwd(src_dir):
            call_command("gitsplit", "--move", "HEAD")
        if has_local_changes:
            self.runcmd(["git", "stash", "pop"], cwd=src_dir)



"""Creates a git tag and allows to interactively rebase onto that tag."""
from __future__ import absolute_import
from dodo_commands.extra.standard_commands import (
    DodoCommand, CommandError
)
from git import Repo


class Command(DodoCommand):  # noqa
    @staticmethod
    def _split_for(x):
        return x + "_split"

    def add_arguments_imp(self, parser):  # noqa
        group = parser.add_mutually_exclusive_group(required=True)
        group.add_argument(
            '--move', dest='move_split_to', action='store',
            help='Move the split point'
        )
        group.add_argument(
            '--rebase', dest='onto', action='store',
            help='Rebase current feature onto origin/onto'
        )
        group.add_argument(
            '--ri', dest='rebase_interactive', action='store_true',
            help='Interactively rebase current feature onto split point'
        )

    def handle_imp(self, move_split_to, onto, rebase_interactive, **kwargs):  # noqa
        repo = Repo(".")
        branch = repo.head.ref.name

        if onto:
            try:
                result = repo.git.rebase(
                    self._split_for(branch),
                    onto=onto,
                    with_extended_output=True
                )
                if result[0] == 0:
                    repo.create_tag(
                        self._split_for(branch),
                        ref=onto,
                        force=True
                    )
            except Exception as e:
                raise CommandError(str(e))

        if move_split_to:
            repo.create_tag(
                self._split_for(branch),
                ref=move_split_to,
                force=True
            )

        if rebase_interactive:
            print("git rebase --interactive -- %s_split" % branch)



"""Run a django-manage command."""
import argparse
from dodo_commands.extra.standard_commands import DodoCommand
from dodo_commands.framework.util import remove_trailing_dashes


class Command(DodoCommand):  # noqa
    decorators = ['docker']

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            'manage_args',
            nargs=argparse.REMAINDER
        )

    def handle_imp(  # noqa
        self, manage_args, *args, **kwargs
    ):
        self.runcmd(
            [
                self.get_config("/DJANGO/python"),
                "manage.py",
            ] + remove_trailing_dashes(manage_args),
            cwd=self.get_config("/DJANGO/src_dir")
        )



"""Print the full configuration."""

import yaml
from . import DodoCommand


class Command(DodoCommand):  # noqa
    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument('--key')

    def handle_imp(self, key, **kwargs):  # noqa
        if key:
            print("%s" % str(self.get_config(key, '')))
        else:
            print(yaml.dump(self.config, default_flow_style=False, indent=4))



# noqa
import argparse
from dodo_commands.extra.standard_commands import DodoCommand

class Command(DodoCommand):  # noqa
    help = ""
    decorators = ['docker']
    docker_options = [
        '--name=autoless',
    ]

    def add_arguments_imp(self, parser):  # noqa
        parser.add_argument(
            'autoless_args',
            nargs=argparse.REMAINDER
        )

    def handle_imp(self, autoless_args, **kwargs):  # noqa
        autoless = self.get_config("/LESS/autoless", "autoless")
        self.runcmd(
            [
                "mkdir",
                "-p",
                self.get_config("/LESS/output_dir")
            ]
        )

        self.runcmd(
            [
                autoless,
                ".",
                self.get_config("/LESS/output_dir")
            ] + autoless_args,
            cwd=self.get_config("/LESS/src_dir")
        )



"""Finds a directory or file inside the current project."""
from . import DodoCommand
from dodo_commands.framework.config import (CommandPath, get_project_dir)
import os
import sys


class Command(DodoCommand):  # noqa
    def add_arguments_imp(self, parser):  # noqa
        """
        Entry point for subclassed commands to add custom arguments.
        """
        parser.add_argument(
            'what',
            nargs='?',
            help=(
                'Print the value of /ROOT/<what>_dir. For example: ' +
                '"dodo which src" prints the value of /ROOT/src_dir.')
        )

        group = parser.add_mutually_exclusive_group()
        group.add_argument(
            '--config',
            action="store_true",
            help='Print where the config file is')
        group.add_argument(
            '--script',
            help='Print where the dodo command script with given name is')

    def handle_imp(self, what, config, script, **kwargs):
        project_dir = get_project_dir()
        if config:
            print(
                os.path.join(
                    project_dir, "dodo_commands", "res", "config.yaml"
                )
            )
        elif script:
            command_path = CommandPath(project_dir)
            for item in command_path.items:
                script_path = os.path.join(
                    item.full_path, script + ".py"
                )
                if os.path.exists(script_path):
                    sys.stdout.write(script_path + "\n")
        elif what:
            sys.stdout.write(self.get_config("/ROOT/%s_dir" % what) + "\n")
        else:
            sys.stdout.write(self.get_config("/ROOT/project_name") + "\n")



# noqa
from dodo_commands.extra.standard_commands import DodoCommand
import os
from plumbum.cmd import whoami


class Command(DodoCommand):  # noqa
    help = "Changes the owner of ${/ROOT/src_dir} to `whoami`:`whoami`"
    decorators = []

    def handle_imp(self, **kwargs):  # noqa
        me = whoami()[:-1]
        src_dir = self.get_config("/ROOT/src_dir")
        self.runcmd(
            [
                "sudo",
                "chown",
                "-R",
                "%s:%s" % (me, me),
                os.path.basename(src_dir)
            ],
            cwd=os.path.dirname(src_dir)
        )



"""Package default_commands."""

from dodo_commands.framework import (
    BaseCommand, CommandPath, get_project_dir
)
from dodo_commands.framework.command_error import CommandError

from importlib import import_module
from plumbum import FG, ProcessExecutionError, local
from dodo_commands.framework.util import query_yes_no


class DodoCommand(BaseCommand):  # noqa
    decorators = []
    _loaded_decorators = None

    @staticmethod
    def _load_decorator(name):
        """Load and return decorator class in module with given name."""
        command_path = CommandPath(get_project_dir())
        command_path.extend_sys_path()
        for item in command_path.items:
            import_path = '%s.decorators.%s' % (
                item.module_path.replace("/", "."), name
            )
            try:
                module = import_module(import_path)
                return module.Decorator()
            except ImportError:
                continue
        return None

    def add_arguments(self, parser):
        """Entry point for subclassed commands to add custom arguments."""
        parser.add_argument(
            '--confirm',
            action='store_true',
            help="Confirm each performed action before its execution"
        )

        parser.add_argument(
            '--echo',
            action='store_true',
            help="Print all commands instead of running them"
        )

        for decorator in self._get_decorators():
            decorator.add_arguments(self, parser)

        self.add_arguments_imp(parser)

    def _get_decorators(self):
        if self._loaded_decorators is not None:
            return self._loaded_decorators

        self._loaded_decorators = list(filter(
            lambda x: x is not None,
            map(self._load_decorator, self.decorators)
        ))
        return self._loaded_decorators

    def add_arguments_imp(self, parser):  # noqa
        pass

    def handle(  # noqa
        self,
        confirm=False,
        echo=False,
        **kwargs
    ):
        self.opt_confirm = confirm
        self.opt_echo = echo

        for decorator in self._get_decorators():
            decorator.handle(self, **kwargs)
        self.handle_imp(**kwargs)

    def handle_imp(self, **kwargs):  # noqa
        raise CommandError("Not implemented")

    def runcmd(  # noqa
        self,
        args,
        cwd=None
    ):
        """
        Decorator to print function call details - parameters names and
        effective values.
        """
        for decorator in self._get_decorators():
            args, cwd = decorator.modify_args(self, args, cwd)

        func = local[args[0]][args[1:]]
        if self.opt_echo:
            print(func)
            return False

        with local.cwd(cwd or local.cwd):
            if self.opt_confirm:
                print("(%s) %s" % (local.cwd, func))
                if not query_yes_no("continue?"):
                    return False
                print()

            try:
                variable_map = self.get_config('/ENVIRONMENT/variable_map', {})
                with local.env(**variable_map):
                    func & FG
                return True
            except ProcessExecutionError:
                print("/nDodo Commands error while running this command:\n\n%s" % func)
                return False



"""Run gitk."""
from dodo_commands.extra.standard_commands import DodoCommand


class Command(DodoCommand):  # noqa
    def handle_imp(  # noqa
        self, **kwargs
    ):
        self.runcmd(
            [
                "gitk",
            ],
            cwd=self.get_config("/ROOT/src_dir"),
        )



"""Show diff for a file in the Dodo Commands system directory."""

from . import DodoCommand
from six.moves import configparser
import os


class Command(DodoCommand):  # noqa
    def add_arguments_imp(self, parser):  # noqa
        """
        Entry point for subclassed commands to add custom arguments.
        """
        parser.add_argument(
            'file',
            help='Show diff for this file'
        )

    def handle_imp(self, file, **kwargs):  # noqa
        project_dir = self.get_config("/ROOT/project_dir", "")

        config = configparser.ConfigParser()
        config.read(os.path.expanduser("~/.dodo_commands/config"))

        diff_tool = config.get("DodoCommands", "diff_tool")
        res_dir = os.path.join(project_dir, "dodo_commands", "res")

        original_file = os.path.realpath(
            os.path.join(
                project_dir, "dodo_commands", "default_project", file
            )
        )
        copied_file = os.path.join(res_dir, file)

        self.runcmd(
            [diff_tool, original_file, copied_file],
            cwd=res_dir
        )
